import time
from flask import request
from prometheus_client import Counter, Histogram
from prometheus_client import start_http_server, make_wsgi_app
from werkzeug.middleware.dispatcher import DispatcherMiddleware

FLASK_REQUEST_ENDPOINT_SENTINEL = "-"

def monitor(app, path="/metrics", http_server=False, port=9090, addr=""):
  FLASK_REQUEST_COUNT = Counter(
    "{}_flask_request_count".format(app.name),
    "{} - Flask Request Count".format(app.name),
    ["method", "endpoint", "http_status"],
  )
  FLASK_REQUEST_LATENCY = Histogram(
    "{}_flask_request_latency_seconds".format(app.name),
    "{} - Flask Request Latency".format(app.name),
    ["method", "endpoint"],
  )

  def before_request():
    request.start_time = time.time()

  def after_request(response):
    request_latency = time.time() - request.start_time
    FLASK_REQUEST_LATENCY.labels(request.method, request.path).observe(
      request_latency
    )
    FLASK_REQUEST_COUNT.labels(
      request.method, request.path, response.status_code
    ).inc()

    return response

  app.before_request(before_request)
  app.after_request(after_request)

  if http_server:
    start_http_server(port, addr)
  else:
    prometheus_app = make_wsgi_app()
    return DispatcherMiddleware(app.wsgi_app, {path: prometheus_app})



from setuptools import setup

setup(
    name='flask-prom',
    version='0.1.1',
    url='https://github.com/noqcks/flask-prom',
    license='BSD',
    author='Benji Visser',
    author_email='benny@noqcks.io',
    maintainer='Benji Visser',
    maintainer_email='benny@noqcks.io',
    description='Prometheus client instrumentation for flask.',
    long_description=__doc__,
    packages=['flask_prom'],
    zip_safe=False,
    platforms='any',
    install_requires=[
        'Flask>=0.10',
        'prometheus-client>=0.0.14'
    ],
    classifiers=[
        'Intended Audience :: Developers',
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: BSD License',

        'Operating System :: OS Independent',
        'Environment :: Web Environment',
        'Topic :: System :: Monitoring',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',

        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    keywords='prometheus monitoring'
)
